//
//	Legato Code Examples
//	--------------------
//
//	Tiny FTP Browser
//
//	Group:		Internet
//	Title:		Tiny FTP Browser
//	Description:	This utility demonstrates the FTP SDK functions as part of Legato.
//	Revised:	12/16/2014
//
//	This software is provided on as "as-is" basis with no warranty implied or otherwise.
//	(c) Novaworks/PSG
//
//	Requires Legato 0.9J or later.
//
//
//	Notes:
//
//	   1.	The FTP channel can fall asleep if there is no activity for a prolong time.
//
//	   2.	The list does not currently have sort provisions.
//
//

								/************************************************/


									/****************************************/
    									/*					*/
									/* ** Dialogs				*/
#include "FTP Utilities.rc"						/* Main Dialog Resources and Defines	*/
    									/*					*/
									/* ** Prototypes			*/
    int 		browse_set_state	(); 			/* Set State				*/
    int 		load_folder		(); 			/* Load the Current Folder		*/
    int			browse_download		();			/* Browse Download Tool			*/
    int			browse_upload		();			/* Browse Upload Tool			*/
    int			browse_delete		();			/* Browse Delete Tool			*/
    int			browse_new_folder	();			/* Browse New Folder Tool		*/
    int			browse_rename		();			/* Browse Rename Tool			*/
    int			query_server		();			/* Query Routine			*/
    									/*					*/
									/* ** Global Data			*/
    string		domain;						/* Domain				*/
    string		u_id, u_pw;					/* User ID and Password			*/
    string		c_path;						/* Current Path				*/
    handle		hFTP;						/* Handle to FTP Connection		*/
									/*  * General				*/
    string		dlg_string_a;					/* Dialog Exchnage String		*/
    string		dlg_string_b;					/* Dialog Exchnage String		*/


								/************************************************/

									/****************************************/
int main() {								/* Main Program Entry			*/
									/****************************************/
    int			rc;						/* Return Code				*/
    									/*					*/
									/* ** Main FTP Load			*/
									/*  * Get Settings			*/
    domain = GetSetting("FTP Server", "Host Name");			/* Get domain				*/
    u_id   = GetSetting("FTP Server", "User ID");			/* Get user ID				*/
    u_pw   = GetSetting("FTP Server", "User Password");			/* Get user password (clear text)	*/
									/*  * Query FTP Info			*/
    rc = query_server();						/* Perform query			*/                                                                        
    if (IsError(rc)) {							/* Cancel or something			*/
      return rc;							/* Exit w/error				*/
      }									/* end error				*/
									/*  * Save Settings			*/
    PutSetting("FTP Server", "Host Name", domain);			/* Save domain				*/
    PutSetting("FTP Server", "User ID", u_id);				/* Save user ID				*/
    PutSetting("FTP Server", "User Password", u_pw);			/* Save user password (clear text)	*/
                                                                        /*  * Log In				*/
    hFTP = FTPOpenConnection(domain, u_id, u_pw);			/* Try to connect			*/
    if (IsError(hFTP)) {						/* Failure				*/
      rc = GetLastError();						/* Get the error code			*/
      MessageBox('X', "Unable to connect to %s 0x%08X", domain, rc);	/* Display error			*/
      return ERROR_SOFT | 1;						/* Exit w/error				*/
      }									/* end error				*/
                                                                        /*  * Browse				*/
    DialogBox("FTPBrowserDlg", "browse_");				/* Enter the browse			*/
    return ERROR_NONE;							/* Exit, no error			*/
    }									/* end routine				*/


								/************************************************/

									/****************************************/
int browse_load() {							/* Load Controls			*/
									/****************************************/
									/*					*/
									/* ** Load Controls			*/
									/*  * Set Up Controls			*/
    DataControlSetColumnPositions(FTP_LIST, 80, 135, 160, 200, 240);	/* Set the positions			*/
    DataControlSetColumnHeadings(FTP_LIST, 				/* Set the headings			*/
    				 "File/Folder Name",			/*   File Name				*/
    				 "Modified",				/*   Date				*/
    				 "Type",				/*   Type				*/
    				 "Size",				/*   Size				*/
    				 "Attributes");				/*   Attributes				*/
    load_folder();							/* Load the current folder		*/
    browse_set_state();							/* Set the control state		*/
    }									/* end routine				*/

									/****************************************/
int browse_action(int c_id, int c_code) {				/* Control Action			*/
									/****************************************/
    string		s1;						/* General				*/
    int			sx;						/* Select Position			*/
    									/*					*/
									/* ** Control Action			*/
									/*  * Download List			*/
    if (c_id == FTP_LIST) {						/* Match ID				*/
      if (c_code == DCN_SELECT_CHANGE) {				/*  o Select Change			*/
	sx = browse_set_state();					/* Perform state set			*/
	if (sx < 0) {							/* Nothing selected			*/
    	  EditSetText(FTP_STATUS, "");					/* Update status			*/
	  return ERROR_NONE;						/* Done					*/
	  }								/* end no select			*/
        s1 = DataControlGetCellText(FTP_LIST, sx, 0);			/* Get the type				*/
    	EditSetText(FTP_STATUS, "Selected: %s", s1);			/* Update status			*/
	return ERROR_NONE;						/* Done					*/
	}								/* end select change			*/
      if (c_code == DCN_DOUBLE_CLICK) {					/*  o Double Click			*/
        sx = DataControlGetRowSelection(c_id);				/* Get the current select		*/
        if (sx < 0) { return ERROR_NONE; }				/* Just leave on error			*/
	s1 = DataControlGetCellText(c_id, sx, 2);			/* Get the type				*/
        if (s1 != "Folder") { return ERROR_NONE; }			/* Just leave on non-folder		*/
	s1 = DataControlGetCellText(c_id, sx, 0);			/* Get the name				*/
        if (s1 == "[..]") {						/*  > Folder Up				*/
          c_path = PathLevelUp(c_path);					/* Move up tree				*/
          }								/* end folder up			*/
        else {								/*  > Folder Name			*/
	  s1 = GetStringSegment(s1, 1);					/* Remove the leading '['		*/
          sx = GetStringLength(s1);					/* Find end position			*/
          s1[sx-1] = 0;							/* Just clip it				*/
	  c_path += s1 + "/";						/* Add path				*/
          }								/* end on folder			*/
									/*  > Reload				*/
        load_folder();							/* Reload folder			*/
	browse_set_state();						/* Perform state set			*/
	return ERROR_NONE;						/* Done					*/
        }								/* end double click			*/
      return ERROR_NONE;						/* Exit w/error				*/
      }									/* end function				*/
									/*  * Refresh				*/
    if (c_id == FTP_REFRESH) {						/* Match ID				*/
      load_folder();							/* Reload folder			*/
      return ERROR_NONE;						/* Exit w/error				*/
      }									/* end function				*/
									/*  * Upload				*/
    if (c_id == FTP_UPLOAD) {						/* Match ID				*/
      browse_upload();							/* Exit w/error				*/
      browse_set_state();						/* Perform state set			*/
      return ERROR_NONE;						/* Done					*/
      }									/* end function				*/
									/*  * Download				*/
    if (c_id == FTP_DOWNLOAD) {						/* Match ID				*/
      return browse_download();						/* Exit w/error				*/
      }									/* end function				*/
									/*  * New Folder			*/
    if (c_id == FTP_NEW_FOLDER) {					/* Match ID				*/
      browse_new_folder();						/* Perform function			*/
      browse_set_state();						/* Perform state set			*/
      return ERROR_NONE;						/* Exit w/error				*/
      }									/* end function				*/
									/*  * Rename				*/
    if (c_id == FTP_RENAME) {						/* Match ID				*/
      browse_rename();							/* Perform delete			*/
      browse_set_state();						/* Perform state set			*/
      return ERROR_NONE;						/* Exit w/error				*/
      }									/* end function				*/
									/*  * Delete				*/
    if (c_id == FTP_DELETE) {						/* Match ID				*/
      browse_delete();							/* Perform delete			*/
      browse_set_state();						/* Perform state set			*/
      return ERROR_NONE;						/* Exit w/error				*/
      }									/* end function				*/
    }									/* end routine				*/


									/****************************************/
int load_folder() {							/* Load the Current Folder		*/
									/****************************************/
    string		s1, s2, s3;					/* Gerenal				*/
    string		list[200][6];					/* File List				*/
    int			folders, files, ix;				/* Folders and Files			*/
    int			rc;						/* Return Code				*/
									/*					*/
                                                                        /* ** Load the Current Folder		*/
									/*  * Initilize				*/
    DataControlResetContent(FTP_LIST);					/* Reset the content			*/
    EditSetText(FTP_PATH, c_path);					/* Set the path				*/
    s2 = c_path + "*.*";						/* Add in wildcard mask			*/
    if (c_path != "") {							/* Not at top				*/
      DataControlAddString(FTP_LIST, "[..]\t\tFolder");			/* Add item				*/
      }									/* end up				*/
									/*  * Load the Folders			*/
    s1 = FTPGetFirstFile(hFTP, s2);					/* Get first file			*/
    while (s1 != "") {							/* Loop until end			*/
									/*  o Get Stats				*/
      list[ix]["modified"] = GetFileModifiedTimeString(hFTP, 		/* Get the modified			*/
      						DS_DATE_TIME);		/*  date time and string		*/
      list[ix]["attributes"] = GetFileAttributeString(hFTP);		/* Get the type of data			*/
      rc = InString(list[ix]["attributes"], "d");			/* Test for folder			*/
      if (rc > 0) {							/* Is a folder				*/
        folders++;							/* Add to count				*/
	s1 = "[" + s1;							/* Add folder				*/
        list[ix]["name"] = s1 + "]";					/* [name]				*/
        list[ix]["type"] = "Folder";					/* Type					*/
        }								/* end folder				*/
      else {								/* Normal file				*/
        list[ix]["name"] = s1;						/* [name]				*/
        list[ix]["type"] = "File";					/* Type					*/
        list[ix]["size"] = IntegerToBinaryUnits(GetFileSize(hFTP), 	/* Get the size				*/
        			ITBU_USE_10THS | ITBU_UNIT_K_BYTES);	/*  and translate			*/
        }								/* end folder				*/
      s1 = list[ix]["name"] + "\t" +					/* Name					*/
      	   list[ix]["modified"] + "\t" +				/*   Date				*/
      	   list[ix]["type"] + "\t" +					/*   Type				*/
      	   list[ix]["size"] + "\t" +					/*   Size				*/
      	   list[ix]["attributes"];					/*   Attributes				*/
      DataControlAddString(FTP_LIST, s1);				/* Add item				*/
      ix++;								/* Next item				*/
      s1 = FTPGetNextFile(hFTP);					/* get the next item			*/
      }									/* end load loop			*/
									/*  * (Dump Out List)			*/
                                                                        /*  ( this is for future sort functions)*/
									/*  * Wrap Up				*/
    s1 = c_path;							/* Copy for display			*/
    if (s1 == "") { s1 = "/   (root)"; }				/* Set top				*/
    EditSetText(FTP_STATUS, "Folder Loaded: %s", s1);			/* Update status			*/
    }									/* end routine				*/


									/****************************************/
int browse_set_state() {						/* Set State				*/
									/****************************************/
    string		s1;						/* General				*/
    int			sx;						/* Select Index				*/
    									/*					*/
    sx = DataControlGetRowSelection(FTP_LIST);				/* Get the current select		*/
    if (sx < 0) {							/* No selection				*/
      ControlDisable(FTP_DOWNLOAD);					/* No download				*/
      ControlDisable(FTP_RENAME);					/* Rename				*/
      ControlDisable(FTP_DELETE);					/* No download				*/
      return sx;							/* Exit					*/
      }									/* end no selection			*/
    s1 = DataControlGetCellText(FTP_LIST, sx, 2);			/* Get the type				*/
    if (s1 != "Folder") { 						/* Non-folder				*/
      ControlEnable(FTP_DOWNLOAD);					/* No download				*/
      ControlEnable(FTP_RENAME);					/* Rename				*/
      ControlEnable(FTP_DELETE);					/* Delete				*/
      }									/* end file				*/
    else {								/* Folder				*/
      ControlDisable(FTP_DOWNLOAD);					/* No download				*/
      s1 = DataControlGetCellText(FTP_LIST, sx, 0);			/* Get the name				*/
      if (s1 == "[..]") {						/* Is the nav item			*/
        ControlDisable(FTP_RENAME);					/* Rename				*/
        ControlDisable(FTP_DELETE);					/* Delete				*/
        }								/* end nav item				*/
      else {								/* Is not nav item			*/
        ControlEnable(FTP_RENAME);					/* Rename				*/
        ControlEnable(FTP_DELETE);					/* Delete				*/
        }								/* end nav item				*/
      }									/* end file				*/
    ControlEnable(FTP_NEW_FOLDER);					/* New Folder				*/
    return sx;								/* Exit w/ index			*/
    }									/* end no selection			*/


								/************************************************/

									/****************************************/
int browse_download() {							/* Browse Download Tool			*/
									/****************************************/
    string		s1, s2,						/* General				*/
    			fnSRC, fnDST;					/* Files				*/
    int			rc,						/* Return Code				*/
    			sx;						/* Select Index				*/
    									/*					*/
    sx = DataControlGetRowSelection(FTP_LIST);				/* Get the current select		*/
    if (sx < 0) { 							/* No selection				*/
      MessageBox('x', "Select a file to download.");			/* Display error			*/
      return ; 								/* Exit					*/
      }									/* end no selection			*/
    s1 = DataControlGetCellText(FTP_LIST, sx, 2);			/* Get the type				*/
    if (s1 == "Folder") { 						/* Non-folder				*/
      MessageBox('x', "Select a file to download.");			/* Display error			*/
      return ; 								/* Exit					*/
      }									/* end no selection			*/
    fnSRC = DataControlGetCellText(FTP_LIST, sx, 0);			/* Get the filename			*/
    fnDST = BrowseSaveFile("Select Download File and Destination", 	/* Select a file			*/
    			   "", fnSRC);					/*   and use the default path and file	*/
    if (fnDST == "") { return ; }					/* Cancel, Nothing to do		*/
    s1 = fnSRC;								/* Keep a copy				*/
    fnSRC = c_path + fnSRC;						/* Get the source with path		*/
    rc = FTPGetFile(hFTP, fnSRC, fnDST);				/* Perform the transfer			*/
    if (IsError(rc)) {							/* Some error				*/
      s1 = GetLastErrorMessage();					/* Get the server message		*/
      EditSetText(FTP_STATUS, "Transfer FAILED: %s", s1);		/* Update status			*/
      s2 = TranslateWindowsError(rc);					/* Try to make sense			*/
      MessageBox('x', "Error %08X downloading:\r\r%s\r%s", rc, s1, s2);	/* Display error			*/
      return ; 								/* Exit					*/
      }									/* end error				*/
    EditSetText(FTP_STATUS, "Transfer complete: %s", s1);		/* Update status			*/
    }									/* end no selection			*/

									/****************************************/
int browse_upload() {							/* Browse Upload Tool			*/
									/****************************************/
    string 		list[200];					/* Upload File List			*/
    string		s1, s2,						/* General				*/
    			fnSRC, fnDST;					/* Files				*/
    int			rc,						/* Return Code				*/
    			ix,						/* General Index			*/
                        size;						/* General				*/
    									/*					*/
									/* ** Upload				*/
									/*  * Select Files			*/
    list = BrowseOpenFiles("Select File for Upload");			/* Perfor browse			*/
    if (IsError(rc)) { return ERROR_CANCEL; }				/* Cancel				*/
									/*  * Make Sure Not Folders		*/
    size = ArrayGetAxisDepth(list);					/* Get the size				*/
    for (ix = 0; ix < size; ix++) {					/* Loop and check			*/
      if (IsFolder(list[ix])) { break; }				/* Stop					*/
      }									/* end loop for test			*/
    if (ix != size) {							/* Found a folder			*/
      MessageBox('x', "Cannot upload folders.");			/* Display message			*/
      return ERROR_NONE;						/* Exit, w/ error			*/
      }									/* end found a folder			*/
									/*  * Upload				*/
    ProgressOpen("Uploading Files...", 0);				/* Open status				*/
									/* Loop and upload			*/
    for (ix = 0; ix < size; ix++) {					/* Loop and check			*/
      rc = ProgressUpdate(ix, size);					/* Upload progress			*/
      if (IsError(rc)) { break; }					/* Cancel				*/
      ProgressSetStatus(1, "Upload file %d of %d", ix+1, size);		/* Display first message		*/
      s1 = GetFilename(list[ix]);					/* Get the filename			*/
      ProgressSetStatus(2, s1);						/* Display first message		*/
      fnDST = c_path + s1;						/* Create destination			*/
      rc = FTPPutFile(hFTP, list[ix], fnDST);				/* Perform upload			*/
      if (IsError(rc)) {						/* Had an error				*/
        s1 = GetLastErrorMessage();					/* Get any message			*/
        break;								/* Stop uploading			*/
	}								/* end upload error			*/
      s2 = AppendWithDelimiter(s2, s1);					/* Create display item			*/
      }									/* end upload loop			*/    
									/*  * Result				*/
    load_folder();							/* Reload folder			*/
    ProgressClose();							/* Clsoe progress			*/
    if (IsError(rc)) {							/* Had an error				*/
      EditSetText(FTP_STATUS, "Error in Transfer: %s", s2);		/* Update status			*/
      return rc;							/* Exit w/error				*/
      }									/* end error				*/
    EditSetText(FTP_STATUS, "Transfer complete: %s", s2);		/* Update status			*/
    }									/* end no selection			*/

									/****************************************/
int browse_delete() {							/* Browse Delete Tool			*/
									/****************************************/
    string		s1, s2,						/* General				*/
    			fnTarget;					/* Target Name				*/
    int			rc,						/* Return Code				*/
    			cx, sx, size,					/* Select Index				*/
                        folder_flag;					/* Is a folder				*/
    									/*					*/
									/* ** Delete				*/
                                                                        /*  * Get Particulars			*/
    sx = DataControlGetRowSelection(FTP_LIST);				/* Get the current select		*/
    if (sx < 0) { 							/* No selection				*/
      MessageBox('x', "Select a file to delete.");			/* Display error			*/
      return ; 								/* Exit					*/
      }									/* end no selection			*/
    folder_flag = FALSE;						/* Default is a not folder		*/
    s1 = DataControlGetCellText(FTP_LIST, sx, 2);			/* Get the type				*/
    if (s1 == "Folder") { 						/* Non-folder				*/
      folder_flag = TRUE;						/* Default is a not folder		*/
      }									/* end no selection			*/
    fnTarget = DataControlGetCellText(FTP_LIST, sx, 0);			/* Get the filename			*/
    									/*  * Verify				*/
    rc = YesNoBox('q', "Permanently delete '%s'", fnTarget);		/* Query				*/
    if (rc != IDYES) { return ERROR_CANCEL; }				/* No delete				*/
                                                                        /*  * Delete File			*/
    if (folder_flag == FALSE) {						/* Deleting file			*/
      fnTarget = c_path + fnTarget;					/* Get the source with path		*/
      rc = FTPDeleteFile(hFTP, fnTarget);				/* Perform delete			*/
      if (IsError(rc)) {						/* Error				*/
	s2 = GetLastErrorMessage();					/* Get the last error message		*/
	MessageBox('x', "Error %08X on delete: \r\r%s", rc, s2);	/* Display error			*/
        return rc;							/* Exit w/error				*/
        }								/* end error				*/
									/*  o Update Display			*/
      EditSetText(FTP_STATUS, "Deleted file: %s", s1);			/* Update status			*/
      }									/* end delete file			*/
      									/*  * Delete Folder			*/
    else {								/* Folder mode				*/
      fnTarget = GetStringSegment(fnTarget, 1);				/* Remove the leading '['		*/
      cx = GetStringLength(fnTarget);					/* Find end position			*/
      fnTarget[cx-1] = 0;						/* Just clip it				*/
      fnTarget = c_path + fnTarget;					/* Get the source with path		*/
      rc = FTPDeleteFolder(hFTP, fnTarget);				/* Perform delete			*/
      if (IsError(rc)) {						/* Error				*/
	s2 = GetLastErrorMessage();					/* Get the last error message		*/
	MessageBox('x', "Error %08X on delete: \r\r%s", rc, s2);	/* Display error			*/
        return rc;							/* Exit w/error				*/
        }								/* end error				*/
									/*  o Update Display			*/
      EditSetText(FTP_STATUS, "Deleted folder: %s", s1);		/* Update status			*/
      }									/* end folder mode			*/
      									/*  * Update Display			*/
    load_folder();							/* Reload folder			*/
    size = DataControlGetRowCount(FTP_LIST);				/* Get the size				*/
    if (sx == size) { sx--; }						/* Adjust if last item			*/
    if (sx >= 0) {							/* Has ok position			*/
      DataControlSetRowSelection(FTP_LIST, sx);				/* Set the selection			*/
      }									/* end selection			*/
    return ERROR_NONE;							/* Exit no error			*/
    }									/* end no selection			*/

									/****************************************/
int browse_new_folder() {						/* Browse New Folder Tool		*/
									/****************************************/
    string		s1, s2,						/* General				*/
    			fnTarget;					/* Target Name				*/
    int			rc;						/* Return Code				*/
    									/*					*/
									/* ** New Folder			*/
                                                                        /*  * Get Name				*/
    rc = DialogBox("FTPNewFolderDlg", "browse_new_folder_");		/* Get the name				*/
    if (IsError(rc)) { return rc; }					/* Cancel				*/
    									/*  * Create				*/
    fnTarget = c_path + dlg_string_a;					/* Create full path			*/
    rc = FTPCreateFolder(hFTP, fnTarget);				/* Perform create			*/
    if (IsError(rc)) {							/* Error				*/
      s2 = GetLastErrorMessage();					/* Get the last error message		*/
      MessageBox('x', "Error %08X on delete: \r\r%s", rc, s2);		/* Display error			*/
      return rc;							/* Exit w/error				*/
      }									/* end error				*/
    c_path = fnTarget + "/";						/* Create new path			*/
    load_folder();							/* Reload folder			*/
    return ERROR_NONE;							/* Exit no error			*/
    }									/* end no selection			*/

									/****************************************/
int browse_new_folder_validate() {					/* Validate Dialog			*/
									/****************************************/
    dlg_string_a = EditGetText(FTP_FOLDER_NAME);			/* Folder name				*/
    if (dlg_string_a == "") {						/* No name				*/
      MessageBox('x', "Folder name required.");				/* Display error			*/
      return ERROR_SOFT | FTP_FOLDER_NAME;				/* Exit w/error on control		*/
      }									/* end name required			*/
    if (IsFilenameValid(dlg_string_a) == FALSE) {			/* Bad name				*/
      MessageBox('x', "Invalid folder name.");				/* Display error			*/
      return ERROR_SOFT | FTP_FOLDER_NAME;				/* Exit w/error on control		*/
      }									/* end name required			*/
    return ERROR_NONE;							/* Controls OK				*/
    }									/* end routine				*/

									/****************************************/
int browse_rename() {							/* Browse Rename Tool			*/
									/****************************************/
    string		s1, s2;						/* General				*/
    int			rc,						/* Return Code				*/
    			sx, cx;						/* Select, Character Index		*/
    									/*					*/
									/* ** Rename Item			*/
									/*  * Get Existing Name			*/
    sx = DataControlGetRowSelection(FTP_LIST);				/* Get the current select		*/
    if (IsError(sx)) { return ERROR_CANCEL; }				/* Nothing selected			*/
    s1 = DataControlGetCellText(FTP_LIST, sx, 0);			/* Get the filename			*/
    s2 = DataControlGetCellText(FTP_LIST, sx, 2);			/* Get the type				*/
    if (s2 == "Folder") { 						/* Non-folder				*/
      s1 = GetStringSegment(s1, 1);					/* Remove the leading '['		*/
      cx = GetStringLength(s1);						/* Find end position			*/
      s1[cx-1] = 0;							/* Just clip it				*/
      }									/* end no selection			*/
                                                                        /*  * Get New Name			*/
    dlg_string_b = s1;							/* Conduit				*/
    rc = DialogBox("FTPRenameDlg", "browse_rename_");			/* Get the new name			*/
    if (IsError(rc)) { return rc; }					/* Cancel				*/
    									/*  * Rename				*/
    s1 = c_path + dlg_string_b;						/* Create full path, existing		*/
    s2 = c_path + dlg_string_a;						/* Create full path, new		*/
    rc = FTPRenameFile(hFTP, s1, s2);					/* Perform rename			*/
    if (IsError(rc)) {							/* Error				*/
      s2 = GetLastErrorMessage();					/* Get the last error message		*/
      MessageBox('x', "Error %08X on rename: \r\r%s", rc, s2);		/* Display error			*/
      return rc;							/* Exit w/error				*/
      }									/* end error				*/
    load_folder();							/* Reload folder			*/
//  DataControlSetRowSelection(FTP_LIST, sx);				/* Reselect				*/	// must fix for position, needs to search
    return ERROR_NONE;							/* Exit no error			*/
    }									/* end no selection			*/

									/****************************************/
int browse_rename_load() {						/* Load Rename Dialog			*/
									/****************************************/
    EditSetText(FTP_EXISTING_NAME, dlg_string_b);			/* Existing Name			*/
    EditSetText(FTP_NEW_NAME, dlg_string_b);				/* New Name				*/
    return ERROR_NONE;							/* Exit no error			*/
    }									/* end no selection			*/

									/****************************************/
int browse_rename_validate() {						/* Validate Rename Dialog		*/
									/****************************************/
    dlg_string_a = EditGetText(FTP_NEW_NAME);				/* New name				*/
    if (dlg_string_a == "") {						/* No name				*/
      MessageBox('x', "New name required.");				/* Display error			*/
      return ERROR_SOFT | FTP_NEW_NAME;					/* Exit w/error on control		*/
      }									/* end name required			*/
    if (IsFilenameValid(dlg_string_a) == FALSE) {			/* Bad name				*/
      MessageBox('x', "Invalid item name.");				/* Display error			*/
      return ERROR_SOFT | FTP_NEW_NAME;					/* Exit w/error on control		*/
      }									/* end name required			*/
    if (dlg_string_a == dlg_string_b) {					/* Same					*/
      MessageBox('x', "Enter in a different name.");			/* Display error			*/
      return ERROR_SOFT | FTP_NEW_NAME;					/* Exit w/error on control		*/
      }									/* end name required			*/
    return ERROR_NONE;							/* Controls OK				*/
    }									/* end routine				*/


								/************************************************/

									/****************************************/
int query_server() {							/* Query for FTP Information		*/
									/****************************************/
    int			rc;						/* Return Code				*/
									/*					*/
    rc = DialogBox("FTPLoginDlg", "login_");				/* Query				*/
    return rc;								/* Exit, no error			*/
    }									/* end routine				*/
    
									/****************************************/
int login_load() {							/* Load Controls			*/
									/****************************************/
    EditSetText(FTP_HOST, domain);					/* Domain name				*/
    EditSetText(FTP_U_ID, u_id);					/* User ID and Password			*/
    EditSetText(FTP_U_PW, u_pw);					/* Password				*/
    }									/* end routine				*/

									/****************************************/
int login_action(int c_id, int c_code) {				/* Control Action			*/
									/****************************************/
    handle		hTest;						/* Handle to FTP Test Connection	*/
    string		s1, s2, s3;					/* General				*/
    int			rc;						/* Return Code				*/
									/*					*/
									/* ** Button Action			*/
									/*  * Test Connection			*/
    if (c_id == FTP_TEST) {						/* Control Match			*/
									/*  o Get Data				*/
      s1 = EditGetText(FTP_HOST);					/* Domain name				*/
      s2 = EditGetText(FTP_U_ID);					/* User ID and Password			*/
      s3 = EditGetText(FTP_U_PW);					/* Password				*/
      if (s1 == "") {							/* Need domain				*/
        MessageBox('X', "Domain Name Required");			/* Display error			*/
	ControlSetFocus(FTP_HOST);					/* Set focus on control			*/
	return ERROR_SOFT;						/* Done					*/
        }								/* end error				*/
									/*  o Disable Controls			*/
      ControlDisable(FTP_TEST);						/* Test Button				*/
      ControlDisable(FTP_HOST);						/* Domain name				*/
      ControlDisable(FTP_U_ID);						/* User ID and Password			*/
      ControlDisable(FTP_U_PW);						/* Password				*/
      ControlDisable(IDOK);						/* OK Button				*/
      ControlDisable(IDCANCEL);						/* Cancel Button			*/
      									/*  o Perform Test			*/
      hTest = FTPOpenConnection(s1, s2, s3);				/* Try to connect			*/
      if (IsError(hTest)) {						/* Failure				*/
        rc = GetLastError();						/* Get the error code			*/
        MessageBox('X', "Unable to connect 0x%08X", rc);		/* Display error			*/
        }								/* end error				*/
      else {								/* Ok					*/
        MessageBox('I', "Connected OK");				/* Display message			*/
        CloseHandle(hTest);						/* Close the connection			*/
        }								/* end ok connection			*/
									/*  o Re-Enable Controls		*/
      ControlEnable(FTP_TEST);						/* Test Button				*/
      ControlEnable(FTP_HOST);						/* Domain name				*/
      ControlEnable(FTP_U_ID);						/* User ID and Password			*/
      ControlEnable(FTP_U_PW);						/* Password				*/
      ControlEnable(IDOK);						/* OK Button				*/
      ControlEnable(IDCANCEL);						/* Cancel Button			*/
      }									/* end test				*/
    return ERROR_NONE;							/* Controls OK				*/
    }									/* end routine				*/

									/****************************************/
int login_validate() {							/* Validate Controls			*/
									/****************************************/
    domain = EditGetText(FTP_HOST);					/* Domain name				*/
    u_id   = EditGetText(FTP_U_ID);					/* User ID and Password			*/
    u_pw   = EditGetText(FTP_U_PW);					/* Password				*/
    return ERROR_NONE;							/* Controls OK				*/
    }									/* end routine				*/
